# coding: utf-8
"""
PYUNO_LOGLEVEL     Valid values are

    * NONE - nothing is logged
    * CALL - the method name of every call is logged
    * ARGS - additionally, the arguments of every call are logged 

NONE is default

PYUNO_LOGTARGET 

    * stdout - logs to stdout (doesnt work on windows within OpenOffifce.org)
    * stderr - logs to stderr (doesnt work on windows within OpenOffifce.org)
    * file-url-prefix(relative urls allowed) - logs to files, which start with
      this string. The pid of the process is appended to the string
      (e.g. file:///c:/temp/bla will write to c:\temp\bla.235 if 235 is the
      pid of the current process)
"""

import uno
import unohelper

from com.sun.star.task import XJob
from com.sun.star.document import DocumentEvent
from com.sun.star.lang import XServiceInfo


from com.sun.star.ui import XContextMenuInterceptor
from com.sun.star.ui.ContextMenuInterceptorAction import (
    IGNORED, CONTINUE_MODIFIED, EXECUTE_MODIFIED)
from com.sun.star.beans import XPropertySet
from com.sun.star.lang import XMultiServiceFactory
from com.sun.star.container import XIndexContainer, XContainer

from com.sun.star.beans import PropertyValue

class JobExecutor(XJob, XServiceInfo, unohelper.Base):
    IMPLE_NAME = "hellug.sgeorgaras.ancientgreek.autoload"
    DOCUMENT_EVENTS = ("OnNew", "OnLoad") 
    #DOCUMENT_EVENTS = ("OnCreate", "OnNew", "OnLoad") 
    """ DOCUMENT_EVENTS = ("OnCreate", 
        "OnLoadFinished", "OnNew", "OnLoad", 
        "OnSave", "OnSaveAs", "OnSaveAsDone", 
        "OnPrepareunload", "OnFocus", "OnUnfocus", 
        "OnPrint", "OnModifyChanged", 
        "OnCopyTo", "OnCopyToDone", 
        "OnViewCreated", "OnPrepareViewClosing", 
        "OnVisAreaChanged", "OnCreate", "OnLoadFinished", 
        "OnSaveAsFailed", "OnSaveFailed", "OnCopyToFailed", 
        "TitleChanged", "OnMailMerge", "OnPageCountChange") """
    APPLICATION_EVENTS = ("OnStartApp", "OnCloseApp", 
        "OnUnload", "OnViewClosed")
    
    def __init__(self, ctx, *args):
        self.ctx = ctx
    # XServiceInfo
    def getImplementationName(self):
        return IMPLE_NAME
    def supportsService(self, name):
        return name == IMPLE_NAME
    def getSupportedServiceNames(self):
        return IMPLE_NAME, 
    def check_context(self, doc, context):
        if len(context) == 0:
            return True
        
        mod_manager = self.ctx.getServiceManager().createInstanceWithContext(
            "com.sun.star.frame.ModuleManager", self.ctx)
        identifier = mod_manager.identify(doc)
        return identifier in context
   
    def createUnoService(self,serviceName):
        """ gets a service from Uno """
        sm = uno.getComponentContext().ServiceManager
        result = sm.createInstance(serviceName)
        #if not result:
        #    raise RuntimeException("\nService not available :\n" + serviceName, uno.getComponentContext())
        return result

    def execute(self, args):
        """ Event executed. """
        event_name = ""
        doc = None
        url = ""
        context = []
        
        for arg in args:
            name = arg.Name
            #print(name)
            if name == "JobConfig": # Arguments
                options = arg.Value
                for option in options:
                    name = option.Name
                    if name == "URL":
                        url = option.Value
                        url.replace("&amp;","&")
                    elif name == "Context":
                        context = option.Value.split(",")
            elif name == "Environment":
                envs = arg.Value
                #print(envs)
                for env in envs:
                    name = env.Name
                    if name == "EnvType" and env.Value == "DOCUMENTEVENT":
                        pass#print(env.Value)
                    elif name == "Model":
                        doc = env.Value
                    elif name == "EventName":
                        event_name = env.Value
        #
        valid = True
        if event_name in self.DOCUMENT_EVENTS:
            if not self.check_context(doc, context):
                valid = False
        else:
            # elif event_name in self.APPLICATION_EVENTS:
            doc = None
        
        if event_name == "OnStartApp":
            print('==== AncientGreek (autoload.py) : Executing AncientGreek.modAccelerator.InitProgram')
            url = "vnd.sun.star.script:AncientGreek.modAccelerator.InitProgram?language=Basic&location=application"
            _mspf = self.createUnoService("com.sun.star.script.provider.MasterScriptProviderFactory")
            sp = _mspf.createScriptProvider("") 
            script = sp.getScript(url)
            """ ev = DocumentEvent(doc, event_name, None, None) """
            ev=None
            a=None
            b=None
            script.invoke((ev,), a, b)
            return

        if valid:
            print('==== AncientGreek (autoload.py) : Run document Auto function?')
            # execute macro
            if not (doc is None):
                if not (doc.supportsService('com.sun.star.script.BasicIDE')):
                    loc=doc.getLocation()
                    if not (loc.startswith("vnd.sun.star.help")):
                        controller = doc.getCurrentController()
                        controller.registerContextMenuInterceptor(DocContextMenuInterceptor(doc))
                        url = "vnd.sun.star.script:AncientGreek.IbycusIcon.onLoad?language=Basic&location=application"
                        if event_name == "OnNew":
                            url = "vnd.sun.star.script:AncientGreek.IbycusIcon.onNew?language=Basic&location=application"
                        sp = doc.getScriptProvider()
                        script = sp.getScript(url)
                        """ ev = DocumentEvent(doc, event_name, None, None) """
                        ev=None
                        a=None
                        b=None
                        print('==== AncientGreek (autoload.py) : Executing AncientGreek.IbycusIcon.%s' % event_name)
                        script.invoke((ev,), a, b)
                    else:
                        print('==== AncientGreek (autoload.py) : Document Location = vnd.sun.star.help - Exiting!!!')
                else:
                    print('==== AncientGreek (autoload.py) : Supports Service = com.sun.star.script.BasicIDE - Exiting!!!')
            else:
                print('==== AncientGreek (autoload.py) : DOC is None!!! - Exiting!!!')

class BaseServiceInfo(XServiceInfo):
    # XServiceInfo
    def getImplementationName(self):
        return self.IMPL_NAME
    def supportsService(self, name):
        return name == self.SERVICE_NAME
    def getSupportedServiceNames(self):
        return self.SERVICE_NAME,
 
 
class ActionTriggerContainer(unohelper.Base, BaseServiceInfo, XIndexContainer, XMultiServiceFactory):
    """ Provides container for items. """
    
    SERVICE_NAME = "com.sun.star.ui.ActionTriggerContainer"
    IMPL_NAME = "hoge"
    
    def __init__(self):
        self.items = []
    
    def append(self, item):
        self.items.append(item)
    
    # XMultiServiceFactory
    def createInstance(self, name):
        pass
    def createInstanceWithArguments(self, name, args):
        pass
    def getAvailableServiceNames(self):
        return ()
    
    # XIndexContainer
    def hasElements(self):
        return len(self.items) != 0
    def getElementType(self):
        return uno.Type("com.sun.star.beans.XPropertyValue")
    def getCount(self):
        return len(self.items)
    def getByIndex(self, n):
        if 0 <= n < len(self.items):
            return self.items[n]
        return None # should be raise IllegalArgumentException
    def replaceByIndex(self, n, item):
        if 0 <= n < len(self.items):
            self.items[n] = item
    def insertByIndex(self, n, item):
        if 0 <= n <= len(self.items):
            self.items.insert(n, item)
    def removeByIndex(self, n):
        if 0 <= n < len(self.items):
            self.items.pop(n)
 
 
class BasePropertySet(unohelper.Base, XPropertySet):
    # XPropertySet
    def addPropertyChangeListener(self, l): pass
    def removePropertyChangeListener(self, l): pass
    def addVetoableChangeListener(self, l): pass
    def removeVetoableChangeListener(self, l): pass
    def getPropertySetInfo(self): pass
    def setPropertyValue(self, name, value): pass
    def getPropertyValue(self, name): pass
 
 
class ActionTriggerItem(BasePropertySet, BaseServiceInfo, object):
    """ Provides item. """
    
    SERVICE_NAME = "com.sun.star.ui.ActionTrigger"
    IMPL_NAME = "hoge"
    
    def __init__(self, command_url, text, help_url, sub_menu=None):
        self.command_url = command_url
        self.text = text
        self.help_url = help_url
        self.sub_menu = sub_menu
    
    def getPropertyValue(self, name):
        if name == "CommandURL":
            return self.command_url
        elif name == "Text":
            return self.text
        elif name == "HelpURL":
            return self.help_url
        elif name == "SubContainer":
            return self.sub_menu
        else:
            return None
    
 
from com.sun.star.ui.ActionTriggerSeparatorType import LINE
 
class DocContextMenuInterceptor(unohelper.Base, XContextMenuInterceptor):
    
    INSERT_SHEET = "slot:26269"
    ACTION_CUT = "slot:5710"
    def __init__(self, doc):
        self.doc = doc

    def getMessage(self , lang_id , msg_id):
        msg = [ u'Convert Beta Code to Unicode' ,
            u'Μετατροπή Beta Code σε Unicode' ,
            u'Convertire da Beta Code a Unicode' ,
            u'Convert Unicode to Beta Code' ,
            u'Μετατροπή Unicode σε Beta Code' ,
            u'Convertire da Unicode a Beta Code' ,
            u'Convert SPIonic to Unicode' ,
            u'Μετατροπή SPIonic σε Unicode' ,
            u'Convertire da SPIonic a Unicode' ,
            u'Convert Unicode to SPIonic' ,
            u'Μετατροπή SPIonic σε Unicode' ,
            u'Convertire da Unicode a SPIonic' ,
            u'Convert Extended SPIonic to Fullwidth Unicode' ,
            u'Εκτεταμένοι SPIonic σε χαρ. μισού/πλήρους πλάτους' ,
            u'Convertire da SPIonic esteso a Unicode a larghezza piena' ,
            u'Convert SGreek to Unicode' ,
            u'Μετατροπή SGreek σε Unicode' ,
            u'Convertire da Sgreek a Unicode' ,
            u'Convert Unicode to SGreek',
            u'Μετατροπή Unicode σε SGreek' ,
            u'Convertire da Unicode a Sgreek' ,
            u'Convert LaserGreek to Unicode' ,
            u'Μετατροπή LaserGreek σε Unicode' ,
            u'Convertire da LaserGreek a Unicode' ,
            u'Convert Unicode to LaserGreek' ,
            u'Μετατροπή Unicode σε LaserGreek' ,
            u'Convertire da Unicode a LaserGreek' ,
            u'Convert GreekKeys to Unicode' ,
            u'Μετατροπή GreekKeys σε Unicode' ,
            u'Convertire da GreekKeys a Unicode' ,
            u'Convert Unicode to GreekKeys' ,
            u'Μετατροπή Unicode σε GreekKeys' ,
            u'Convertire da Unicode a GreekKeys' ,
            u'Convert Ismini to Unicode' ,
            u'Μετατροπή Ismini σε Unicode' ,
            u'Convertire da Ismini a Unicode' ,
            u'Convert Unicode to Ismini' ,
            u'Μετατροπή Unicode σε Ismini' ,
            u'Convertire da Unicode a Ismini' ,
            u'Convert WinGreek to Unicode' ,
            u'Μετατροπή WinGreek σε Unicode' ,
            u'Convertire da WinGreek a Unicode' ,
            u'Convert Unicode to WinGreek' ,
            u'Μετατροπή Unicode σε WinGreek' ,
            u'Convertire da Unicode a WinGreek' ,
            u'Convert Son of WinGreek to Unicode' ,
            u'Μετατροπή Son of WinGreek σε Unicode' ,
            u'Convertire da Son of WinGreek a Unicode' ,
            u'Convert Unicode to Son of WinGreek' ,
            u'Μετατροπή Unicode σε Son of WinGreek' ,
            u'Convertire da Unicode a Son of WinGreek' ]
        act_id = int(msg_id * 3 + lang_id)
        try:
            ret_val = msg[act_id]
        except:
            ret_val = ''
        return ret_val

    def notifyContextMenuExecute(self, ev):
        try:
            cont = ev.ActionTriggerContainer
        except:
            return IGNORED
        try:
            if not self.doc.supportsService("com.sun.star.text.GenericTextDocument"):
                return IGNORED
            if (not (cont.getByIndex(0).CommandURL == "GoTo")) and self.canWorkHere(cont):
                item = cont.createInstance("com.sun.star.ui.ActionTriggerSeparator")
                item.SeparatorType = LINE
                #cont.insertByIndex(8, item)
                theLang = self.getLanguage()
                LANGS = { "en-Us":0 , "el":1 , "it":2 }
                try:
                    lang_id = LANGS[theLang]
                except:
                    lang_id = 0
                items = ActionTriggerContainer()
                items.insertByIndex(0, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.BetaCodeToUnicode", self.getMessage(lang_id,0) , "", None))
                items.insertByIndex(1, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.UnicodeToBetaCode", self.getMessage(lang_id,1), "", None))
                items.insertByIndex(2, item)
                items.insertByIndex(3, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.SPIonicToUnicode", self.getMessage(lang_id,2), "", None))
                items.insertByIndex(4, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.UnicodeToSPIonic", self.getMessage(lang_id,3), "", None))
                items.insertByIndex(5, ActionTriggerItem("macro:///AncientGreek.SPIonic.ExtendedSPIonicToWideCharsWrapper" , self.getMessage(lang_id,4) , "" , None))
                items.insertByIndex(6, item)
                items.insertByIndex(7, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.SGreekToUnicode", self.getMessage(lang_id,5), "", None))
                items.insertByIndex(8, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.UnicodeToSGreek", self.getMessage(lang_id,6), "", None))
                items.insertByIndex(9, item)
                items.insertByIndex(10, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.LaserGreekToUnicode", self.getMessage(lang_id,7), "", None))
                items.insertByIndex(11, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.UnicodeToLaserGreek", self.getMessage(lang_id,8), "", None))
                items.insertByIndex(12, item)
                items.insertByIndex(13, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.GreekKeysToUnicode", self.getMessage(lang_id,9), "", None))
                items.insertByIndex(14, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.UnicodeToGreekKeys", self.getMessage(lang_id,10), "", None))
                items.insertByIndex(15, item)
                items.insertByIndex(16, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.IsminiToUnicode", self.getMessage(lang_id,11), "", None))
                items.insertByIndex(17, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.UnicodeToIsmini", self.getMessage(lang_id,12), "", None))
                items.insertByIndex(18, item)
                items.insertByIndex(19, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.WinGreekToUnicode", self.getMessage(lang_id,13), "", None))
                items.insertByIndex(20, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.UnicodeToWinGreek", self.getMessage(lang_id,14), "", None))
                items.insertByIndex(21, item)
                items.insertByIndex(22, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.SonOfWinGreekToUnicode", self.getMessage(lang_id,15), "", None))
                items.insertByIndex(23, ActionTriggerItem("macro:///AncientGreek.CharacterConversion.UnicodeToSonOfWinGreek", self.getMessage(lang_id,16), "", None))
                item = ActionTriggerItem("GoTo", "AncientGreek", "", items)
                #cont.insertByIndex(cont.getCount(), item)
                cont.insertByIndex(0, item)
                
                return EXECUTE_MODIFIED
        except:
            return IGNORED
        return IGNORED

    def canWorkHere(self , cont):
        i = 0
        while i < cont.getCount():
            try:
                if cont.getByIndex(i).CommandURL == self.ACTION_CUT or cont.getByIndex(i).CommandURL == ".uno:Cut":
                    return True
            except:
                pass
            i+=1
        return False

    def get_instance(self,service_name):
        """ gets a service from Uno """
        sm = uno.getComponentContext()
        ctx = sm.getServiceManager()
        try:
            service = ctx.createInstance(service_name)
        except:
            service = NONE
        return service
    def getLanguage(self):
        try:
            oProvider = 'com.sun.star.configuration.ConfigurationProvider'
            oAccess   = 'com.sun.star.configuration.ConfigurationAccess'
            oConfigProvider = self.get_instance(oProvider)
            oProp = PropertyValue()
            oProp.Name = 'nodepath'
            oProp.Value = 'org.openoffice.Office.Linguistic/General'
            properties = (oProp,)
            key = 'UILocale'
            oSet = oConfigProvider.createInstanceWithArguments(oAccess, properties)
            if oSet and (oSet.hasByName(key)):
                ooLang = oSet.getPropertyValue(key)
        except:
            ooLang = ""
        if not (ooLang and not ooLang.isspace()):
            try:
                oProp.Value = '/org.openoffice.Setup/L10N'
                properties = (oProp,)
                key = 'ooLocale'
                oSet = oConfigProvider.createInstanceWithArguments(oAccess, properties)
                if oSet and (oSet.hasByName(key)):
                    ooLang = oSet.getPropertyValue(key)
            except:
                ooLang = ""
        return ooLang

g_ImplementationHelper = unohelper.ImplementationHelper()
g_ImplementationHelper.addImplementation(
    JobExecutor,
    "hellug.sgeorgaras.ancientgreek.autoload", ("hellug.sgeorgaras.ancientgreek.autoload",),)

